/*
 * Copyright (c) 2019 Adrian Siekierka
 *
 * This file is part of Redstone Minus Redstone.
 *
 * Redstone Minus Redstone is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Redstone Minus Redstone is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Redstone Minus Redstone.  If not, see <http://www.gnu.org/licenses/>.
 */

package pl.asie.redstoneminus;

import net.minecraft.block.BlockRedstoneWire;
import net.minecraft.block.state.IBlockState;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.energy.CapabilityEnergy;
import net.minecraftforge.energy.IEnergyStorage;

final class RedstoneMinusHelpers {
	private RedstoneMinusHelpers() {

	}

	static void tick(World world, TileEntity tile) {
		if (world == null || tile == null || tile.func_145837_r()) {
			return;
		}

		try {
			for (EnumFacing facing : EnumFacing.field_82609_l) {
				BlockPos posOpp = tile.func_174877_v().func_177972_a(facing);
				IBlockState state = world.func_180495_p(posOpp);
				int power;

				if (state.func_177230_c() instanceof BlockRedstoneWire) {
					power = state.func_177229_b(BlockRedstoneWire.field_176351_O);
				} else {
					power = state.func_185911_a(world, posOpp, facing);
				}

				if (power <= 0) continue;

				if (power > 15) {
					power = RedstoneMinusRedstone.powerRamp[14];
				} else {
					power = RedstoneMinusRedstone.powerRamp[power - 1];
				}

				if (power > 0) {
					if (tile.hasCapability(CapabilityEnergy.ENERGY, facing)) {
						IEnergyStorage storage = tile.getCapability(CapabilityEnergy.ENERGY, facing);
						if (storage != null) {
							storage.receiveEnergy(power, false);
						}
					}
				}
			}
		} catch (Exception e) {
			RedstoneMinusRedstone.logger.warn("Exception while applying redstone power @ " + tile.func_174877_v().toString(), e);
		}
	}
}
