/*
 * Copyright (c) 2019 Adrian Siekierka
 *
 * This file is part of Redstone Minus Redstone.
 *
 * Redstone Minus Redstone is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Redstone Minus Redstone is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Redstone Minus Redstone.  If not, see <http://www.gnu.org/licenses/>.
 */

package pl.asie.redstoneminus.core;

import net.minecraft.launchwrapper.IClassTransformer;
import org.objectweb.asm.*;

public class RedstoneMinusTransformer implements IClassTransformer {
	public static class MyClassVisitor extends ClassVisitor {
		public MyClassVisitor(int api, ClassVisitor cv) {
			super(api, cv);
		}

		@Override
		public MethodVisitor visitMethod(int access, String name, String desc,
		                                 String signature, String[] exceptions) {
			return new MyMethodVisitor(api, super.visitMethod(access, name, desc, signature, exceptions));
		}
	}

	public static class MyMethodVisitor extends MethodVisitor {
		public MyMethodVisitor(int api, MethodVisitor mv) {
			super(api, mv);
		}

		@Override
		public void visitMethodInsn(int opcode, String owner, String name,
		                            String desc, boolean itf) {
			if (Opcodes.INVOKEVIRTUAL == opcode && "net/minecraft/block/Block".equals(owner)
					&& "canConnectRedstone".equals(name) && "(Lnet/minecraft/block/state/IBlockState;Lnet/minecraft/world/IBlockAccess;Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/EnumFacing;)Z".equals(desc)) {
				super.visitMethodInsn(Opcodes.INVOKESTATIC, "pl/asie/redstoneminus/RedstoneMinusHooks", "canConnectRedstone",
						"(Lnet/minecraft/block/Block;Lnet/minecraft/block/state/IBlockState;Lnet/minecraft/world/IBlockAccess;Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/EnumFacing;)Z", false);
			} else {
				super.visitMethodInsn(opcode, owner, name, desc, itf);
			}
		}
	}

	@Override
	public byte[] transform(String name, String transformedName, byte[] basicClass) {
		if (transformedName.startsWith("pl.asie.redstoneminus")) {
			return basicClass;
		}

		ClassWriter writer = new ClassWriter(Opcodes.ASM5);
		new ClassReader(basicClass).accept(new MyClassVisitor(Opcodes.ASM5, writer), 0);

		return writer.toByteArray();
	}
}
